<template>
    <div class="container" v-if="!!currentTask.status">
      <h1 class="fs-3 fl-4">Submit Image List To Analyze</h1>

      <div class="row mt-8 fl-5">
        <div class="col-sm-12 col-xl-9">
          <transition name="slide" mode="out-in" appear>
            <div class="mt-2 fl-4" v-if="!filelist.length && !submitted">
              <div class="mb-6 fl-5" v-if="hasCurrentTask" v-html="contents ['subtext:TUNED']"></div>
              <div class="mb-6 fl-5" v-else v-html="contents ['subtext:TUNING']"></div>
              <dropzone @change="updateFileList" :allowFile="true" :mimeTypes="['image/']" />
            </div>
          </transition>

          <transition name="fade" mode="out-in" appear>
            <div class="mt-2 fl-4" v-if="!filelist.length && submitted">
              <div class="complete-sticker">
                <span class="bg-green-darken-2 fs-4 fw-bold px-5 py-2">Submit Completed</span>
              </div>
            </div>
          </transition>

          <transition name="slide" mode="out-in" appear>
            <div class="mt-2 fl-4 rounded-2" v-if="filelist.length">
              <div class="mb-3 row" v-if="!submitted">
                <div class="col-sm-8 text-end">
                  <div v-if="!!filterKeyword" class="fs-14px pt-1">Filtered <b class="text-grey">{{ filterResult.length.format () }}</b> of {{ (submitted ? totalFiles : filelist.length).format () }}</div>
                </div>
                <div class="col-sm-4 text-end">
                  <input type="text" class="form-control form-control-sm" placeholder="Filter file name" v-model="filterKeyword">
                </div>
              </div>

              <table class="table table-dark table-striped table-hover">
                <thead>
                  <tr>
                    <th scope="col" class="text-end">#</th>
                    <th scope="col">File Name</th>
                    <th scope="col" class="text-end">File Size</th>
                    <th scope="col" class="text-end">Remove</th>
                  </tr>
                </thead>
                <tbody>
                  <tr v-for="(file, index) in getBegins ()" :key="index">
                    <td class="text-end">{{ index + 1 }}</td>
                    <td>{{ file.relativePath || file.name }}</td>
                    <td class="text-end">{{ (file.size / 1024 / 1024).format ('usd') }}M</td>
                    <td class="text-end"><a href="#" class="btn text-danger btn-sm" @click="removeItem (file)"><i class="bi bi-trash3-fill bi-sm"></i></a></td>
                  </tr>
                  <tr v-if="filterResult.length > 10">
                    <td class="text-center" colspan="4"><i class="bi bi-three-dots-vertical"></i></td>
                  </tr>
                  <tr v-for="(file, index) in getEnds ()" :key="index">
                    <td class="text-end">{{ filterResult.length - 4 + index }}</td>
                    <td>{{ file.relativePath || file.name }}</td>
                    <td class="text-end">{{ (file.size / 1024 / 1024).format ('usd') }}M</td>
                    <td class="text-end"><a href="#" class="btn text-danger btn-sm" @click="removeItem (file)"><i class="bi bi-trash3-fill bi-sm"></i></a></td>
                  </tr>
                </tbody>

                <tfoot class="table-dark">
                    <tr>
                      <td colspan="2" class="pl-3 fs-16px">Total <b class="text-warning">{{ (submitted ? totalFiles : filelist.length).format () }}</b> Images are in the Submission List</td>
                      <td class="text-end fs-16px"><span class="text-warning fw-bold">{{ totalFileSize }}</span></td>
                      <td class="text-end"><button class="btn btn-danger btn-sm" @click="removeAll">All <i class="bi bi-trash3-fill bi-sm"></i></button></td>
                    </tr>
                </tfoot>
              </table>

              <div class="mt-4 fl-4 actions">
                <div v-if="hasCurrentTask">
                  <button class="mr-4 btn btn-primary" @click="taskName.showTaskNameDialog (currentTask)">Run Paid Analysis<i class="bi bi-play-circle mx-2"></i></button>
                </div>
                <div v-else>
                  <div class="my-8" v-html="contents ['understood']"></div>
                  <button class="mr-4 btn btn-secondary" @click="router.back ()" :disabled="submitted">Back</button>
                  <button class="mr-4 btn btn-primary" @click="understood ()" :disabled="submitted">Understood, Continue</button>
                </div>
              </div>
            </div>
        </transition>
        </div>
      </div>

      <div class="modal" tabindex="-1" id="progressModalStep2" data-bs-backdrop="static">
        <div class="modal-dialog">
          <div class="modal-content">
            <div class="modal-body">
              <div class="col-12">
                <div class="text-secondary mb-2">
                  <b>Submitting Image List</b> <span class="ml-0 text-green-darken-4">{{ registeredFiles.format () }} / {{ totalFiles.format () }} ({{ registerProgress }}%)</span>
                </div>
                <div class="progress">
                  <div class="bg-success progress-bar progress-bar-striped" role="progressbar" :style="'width: ' + registerProgress + '%'" :aria-valuenow="registerProgress" aria-valuemin="0" aria-valuemax="100"></div>
                </div>

                <div class="text-grey-darken-2 mt-4" style="font-size: 14px;">
                  Please, be patient. Do not close tab nor refresh current page.
                </div>
              </div>
            </div>
          </div>
        </div>
      </div>

      <task-name ref="taskName" mode="start" :on-ok="startTask"></task-name>

    </div>



</template>




<style scoped>
.reference {
  color: #9f9f9f;
  font-size: 15px;
}

table th, table td {
  font-size: 13px;
}

.complete-sticker {
  position: absolute;
  top: 40%;
  left: 50%;
  transform:  translate(-50%, -50%) rotate( -30deg );
  opacity: 0.8;
}
</style>






<script setup>
import Dropzone from "/atila-vue/components/dropzone.vue"
import TaskName from "./components/task-name.vue"

const taskName = ref (null)
const simulate = false
const router = useRouter ()
const { state } = useStore ()
const currentTask = ref ({})

const contents = ref ({})
const filelist = ref ([])
const totalBytes = ref (0)
const hasCurrentTask = ref (false)
const submitted = ref (false)
const totalFiles = ref (0)
const registeredFiles = ref (0)

const totalFileSize = computed (() => {
  const mb = totalBytes.value / 1024 / 1024
  if (mb < 10) {
    return `${mb.toFixed (1)} Mb`
  }
  if (mb < 10000) {
    return `${mb.toFixed (1)} Mb`
  }
  return `${(mb / 1024).toFixed (1)} Gb`
})

const registerProgress = computed (() => {
  return Math.round ((registeredFiles.value / totalFiles.value) * 100)
})

function getBegins () {
  if (filterResult.value.length <= 10) {
    return filterResult.value
  }
  return filterResult.value.slice (0, 5)
}

function getEnds () {
  if (filterResult.value.length <= 10) {
    return []
  }
  return filterResult.value.slice (filterResult.value.length - 5, filterResult.value.length)
}


async function understood () {
  submitted.value = true
  await registerFileList ()
  router.push ("/tasks/settings")
}

async function startTask (taskName) {
  await registerFileList ()
  await backend.patch ('UPDATE_TASK', {
    task_id: currentTask.value.id,
    status: 'INPROGRESS', name: taskName
  })

  await backend.delete ('DELETE_OUTPUTS', {task_id: currentTask.value.id}) // remove test run files
  currentTask.value.status = 'INPROGRESS'
  currentTask.value.name = taskName
  state.$ls.set (state.$g.taskKey, currentTask.value)
  await frontend.refresh_task ()
  router.push ({path: '/tasks/analysis', query: {reload: 'yes'}})
}

async function insuffientDiskError () {
  bootstrap.modal ({
    title: "Insuffient Disk Space",
    text: `Please ensure that you have sufficient free disk space, which must exceed the total size of the files you intend to submit.`,
    closeButtonText: 'Confirm',
  })
  await state.$idb.reinitialize ()
  router.push ('/tasks')
}


async function registerFileList () {
  const progressModal = new bootstrap.Modal(document.getElementById ('progressModalStep2'))
  totalFiles.value = filelist.value.length
  registeredFiles.value = 0
  progressModal.show ()

  try {
    await state.$idb.clear ('failed')
    await state.$idb.clear ('results')
    await state.$idb.clear ('queue')
  } catch (e) {
    if (e.name == 'InvalidStateError') {
      progressModal.hide ()
      bootstrap.modal ({
        title: "Unable To Proceed",
        text: contents.value ['modal:database-locked'],
        closeButtonText: 'Confirm',
      })
      throw (e)
    }
  }

  const data = []
  while (filelist.value.length) {
    const file = filelist.value.shift ()
    data.push ([file.relativePath || file.name, {file, status: 'WAITING'}])
    if (data.length == 256 || (filelist.value.length == 0 && data.length)) {
      try {
        for await (let batch of state.$idb.setAll ('queue', data)) {
          registeredFiles.value += batch
        }
        data.splice (0, data.length)
      } catch (e) {
        progressModal.hide ()
        await insuffientDiskError ()
        throw (e)
      }
    }
  }
  registeredFiles.value = totalFiles.value

  if (currentTask.value.status == 'NOTCREATED') {
    const setting = JSON.stringify (currentTask.value.setting)
    let task = {name: 'Task', status: 'TUNING', queued: totalFiles.value, setting, task_type: currentTask.value.task_type}
    let resp = await backend.post ('NEW_TASK', task)
    resp = await backend.get ('GET_DELETE_TASK', {task_id: resp.data.id})
    task = resp.data
    task.setting = currentTask.value.setting
    state.$ls.set (state.$g.taskKey, task)
  } else {
    await backend.patch ('UPDATE_TASK', {
      task_id: currentTask.value.id,
      queued: totalFiles.value
    })
    currentTask.value.queued = totalFiles.value
    state.$ls.set (state.$g.taskKey, currentTask.value)
  }
  progressModal.hide ()
}


function updateFileList (newFielList, notImages, bytes) {
  function _continue () {
    if (notImages) {
      bootstrap.flash ({
        text: `${notImages.format ()} file(s) are ignored becasue they do not look like image file.`,
        class_: 'bg-red',
        icon: 'warning',
        autoHide: false
      })
    }
    newFielList.sort ((a, b) => {
      if (a.relativePath > b.relativePath) return 1
      if (a.relativePath < b.relativePath) return -1
      return 0
    })
    filelist.value = newFielList
    totalBytes.value = bytes
    filterByKeyword ('')
  }

  if (currentTask.value.status == 'TUNED') {
    _continue ()
    return
  }

  bootstrap.modal ({
    title: "Confirmation Required",
    text: contents.value [newFielList.length <= state.$g.maxTestAnalysis ? 'modal:tuning-mode' : 'modal:analysis-mode'],
    closeButtonText: null,
    icon: "question",
    buttons: [
      {text: "Understood, Continue", class_: "btn-primary", run: _continue}
    ]
  })
}


const filterKeyword = ref ('')
const filterResult = ref ([])
function filterByKeyword (newVal) {
  if (newVal.length < 2) {
    filterResult.value = filelist.value
    return
  }
  const kw = newVal.toLowerCase ()
  filterResult.value = filelist.value.filter ((file) => (file.relativePath || file.name).toLowerCase ().indexOf (kw) != -1)
}

watch (filterKeyword, (newVal) => {
  filterByKeyword (newVal)
})





function processRemoveAll () {
  filelist.value.splice (0, filelist.value.length)
  filterByKeyword ('')
}

function removeAll () {
  bootstrap.modal ({
    title: "Remove All",
    text: contents.value ['modal:remove-all'],
    closeButtonText: "Cancel",
    icon: "question",
    buttons: [
      {text: "Remove All", class_: "btn-danger", run: processRemoveAll}
    ]
  })
}

function removeItem (target) {
  const index = filelist.value.findIndex ((file) => file.name == target.name)
  const name = filelist.value [index].name
  bootstrap.flash ({
      text: `<b>${name}</b> has been removed.`,
      class_: 'alert-info',
      icon: 'info',
      autoHide: 3000,
    })
  filelist.value.splice (index, 1)
  filterByKeyword (filterKeyword.value)
}

onBeforeMount (async () => {
  contents.value = await frontend.fetch_contents ({
    totalFileSize: totalFileSize.value
  })
})

onActivated (async () => {
  const taskKey = await frontend.wait_for_true ('state.$g.taskKey')
  currentTask.value = state.$ls.get (taskKey)
  hasCurrentTask.value = false
  totalFiles.value = 0
  registeredFiles.value = 0

  if (simulate) {
    const files = []
    for (let i = 0; i < 600; i++) {
     files.push ({
        name: `P${i.zfill (4)}-0650-E1-4_600_s008220000.jpg`,
        relativePath: `mydir/P${i.zfill (4)}-0650-E1-4_600_s008220000.jpg`,
        size: 520000000
      })
    }
    updateFileList (files)
  }

  if (currentTask.value.setting.options.road_length_m == 0) {
    router.push ('/tasks')
    return bootstrap.modal ({
      title: "Task is not created",
      text: `There is no active task. Create a new task.`,
      closeButtonText: 'Confirm',
    })
  } else if (!(currentTask.value.status == 'NOTCREATED' || currentTask.value.status == 'TUNED')) {
    router.push ('/tasks')
    return bootstrap.modal ({
      title: "Task is alreay in progress",
      text: `You have submitted image list to analysis. If you want to start new task, complete, delete or cancel active task.`,
      closeButtonText: 'Confirm',
    })
  }

  if (currentTask.value.status == 'TUNED') {
    hasCurrentTask.value = true
  }
})


onDeactivated (() => {
  frontend.set_title ()
  submitted.value = false
  processRemoveAll ()
})
</script>

<script>
export default {
  beforeRouteEnter (to, from, next) {
    frontend.permission.required (['user'], {path: '/members/signin'}, {to, from, next})
  }
}
</script>